@@ -1,6 +1,4 @@
     
-
-
     #! :param means parameter
 def search_all(lists:list,value,index_range:tuple):
     """
@@ -142,32 +140,65 @@ class Flexi:
         item = list(self.dictionary.items())[index]
         return item
 
-    def d_exchange(self, no_of_exchanges=1,return_type="no_update"):
+    def d_exchange(self, no_of_exchanges=1,entanglement = True,return_type="no_update"):
         """
         It exchanges the keys and values of a dictionary
-        :param no_of_exchanges: the number of times the dictionary is exchanged, defaults to 1 (optional)
-        :param return_type: if you want to update the dictionary or not, defaults to no_update (optional)
-        :return: the exchanged dictionary.
+        
+        :param no_of_exchanges: the number of times the dictionary is exchanged, defaults to 1 (optional),
+        defaults to 1 (optional)
+        :param entanglement: if True, it will exchange the dictionary even if it has identical values,
+        defaults to True (optional) else it will show error message.
+        :param return_type: if you want to update the dictionary or not, defaults to no_update (optional),
+        defaults to no_update (optional)
+        :return: The exchanged dictionary is being returned.
         """
         if isinstance(no_of_exchanges, int) == False:
             raise Exception("type(no_of_exchanges) == int")
+        if entanglement not in [True,False]:
+            raise Exception("entanglement must be True or False")
         if return_type not in ["no_update", "update"]:
             raise Exception("return_type should be either update or no_update")
+
         
         exchanged_dictionary = self.dictionary
         if no_of_exchanges % 2 != 0:
-            exchanged_dictionary = {value: key for key,
+            if entanglement == False:
+                exchanged_dictionary = {value: key for key,
                                     value in self.dictionary.items()}
-        
-        if len(self.dictionary) == len(exchanged_dictionary):
+                if len(self.dictionary) == len(exchanged_dictionary):
+                    if return_type == "update":
+                        self.dictionary.clear()
+                        self.dictionary.update(exchanged_dictionary)
+                    if return_type == "no_update":
+                        return exchanged_dictionary
+                else :
+                    raise Exception("the dictionary has identical values so it cannot be exchanged")
+            if entanglement == True:
+                exchanged_dictionary = self.dictionary.copy()
+                new_object = Flexi(exchanged_dictionary)
+                index_list = []
+                for values in self.dictionary.values():
+                    key_list = Flexi.d_get(self,values)
+                    if len(key_list) > 1:
+                        exchanged_dictionary.update({key_list:exchanged_dictionary.get(key_list[0])})
+                        index_list = [new_object.d_index(keys) for keys in key_list]
+                if return_type == "no_update":
+                    if len(index_list)!=0:
+                        new_exchanged_dictionary = new_object.d_remove(index_list,return_type="no_update")
+                    return {value: key for key,value in new_exchanged_dictionary.items()}
+                
+                elif return_type == "update":
+                    if len(index_list)!=0:
+
+                        new_object.d_remove(index_list,return_type="update")
+                    self.dictionary.clear()
+                    self.dictionary.update({value: key for key,value in exchanged_dictionary.items()})
+        else:
             if return_type == "update":
-                self.dictionary.clear()
-                self.dictionary.update(exchanged_dictionary)
+                pass
             if return_type == "no_update":
-                return exchanged_dictionary
-        else :
-            raise Exception("the dictionary has identical values so it cannot be exchanged")
-        
+                return self.dictionary
+
 
     def d_reverse(self, no_of_times=1, return_type="no_update"):
         """
@@ -268,22 +299,27 @@ class Flexi:
             self.dictionary.clear()
             self.dictionary.update(inserted_dictionary)
 
-    def d_remove(self, index_list: list):
+    def d_remove(self, index_list: list,return_type = "no_update"):
         """
         It takes the dictionary, copies it, and then removes the key at the index
         
-        :param index: the index of the item to be removed
-        :type index: int
+        :param index_list: list
+        :type index_list: list
+        :param return_type: It's either "no_update" or "update". If it's "no_update", it returns a new
+        dictionary with the key removed. If it's "update", it updates the dictionary, defaults to no_update
+        (optional)
         :return: A new dictionary with the key removed.
         """
+        if return_type not in ["no_update","update"]:
+            raise Exception("return_type should be either no_update or update")
         new_dictionary = self.dictionary.copy()
-        new_object = Flexi(new_dictionary)
-        index_list.sort()
-        if index_list == list(range(index_list[-1]+1)):
-            return new_object.d_split(index_list[-1]+1)[1]
         for index in index_list:
-            new_dictionary.pop(Flexi.d_key(self, index))
-        return new_dictionary
+            new_dictionary.pop(Flexi.d_key(self,index))
+        if return_type == "no_update":
+            return new_dictionary
+        elif return_type == "update":
+            self.dictionary.clear()
+            self.dictionary.update(new_dictionary)
 
     def d_replace(self, index: int, key, value, return_type="no_update"):
         """
@@ -657,10 +693,7 @@ class Flexi:
                     if common_item_list.count(items) == 0:
                         common_item_list.append(items)
         return tuple(common_item_list)
-from time import perf_counter
-start = perf_counter()
-x = Flexi({"a":1, "b":2,"c":3,"d":4,"e":5,"f":6,"g":7})
-print(x.d_replace(1,"j","k",return_type="update"))
-stop = perf_counter()
-print(stop - start)
+
+
+
 
